@@ -237,6 +237,20 @@ async def filter_and_transform_events_for_client(
         # to the cache!
         cloned = clone_event(filtered)
         cloned.unsigned[EventUnsignedContentFields.MEMBERSHIP] = user_membership
+        if storage.main.config.experimental.msc4354_enabled:
+            sticky_duration = cloned.sticky_duration()
+            if sticky_duration:
+                now_ms = storage.main.clock.time_msec()
+                expires_at = (
+                    # min() ensures that the origin server can't lie about the time and
+                    # send the event 'in the future', as that would allow them to exceed
+                    # the 1 hour limit on stickiness duration.
+                    min(cloned.origin_server_ts, now_ms) + sticky_duration.as_millis()
+                )
+                if expires_at > now_ms:
+                    cloned.unsigned[EventUnsignedContentFields.STICKY_TTL] = (
+                        expires_at - now_ms
+                    )
 
         return cloned
 
